#!/usr/bin/perl

package ACL2::FiltroFicheros;

=head1 NAME

ACL2::FiltroFicheros - ME<oacute>dulo de conveniencia para filtrado de ficheros

=cut

#
# Módulo de conveniencia para filtrado de ficheros
# Copyright (C) 2008 Antonio García Domínguez
# $Id$
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;

use Carp;

use XML::Writer;    # XML
use IO::File;       # Manejador de ficheros - necesario para salida a fichero con XML::Writer
use Cwd 'realpath'; # Manejador de directorios, rutas y nombres de fichero
use File::ShareDir 'dist_dir'; # Directorio de ficheros compartidos - contiene el DTD
use FindBin;        # Obtiene la ruta del script

use ACL2::Localizacion;
use ACL2::Tags;
use ACL2::Procesador;

my $lh = ACL2::Localizacion->get_handle();

###################################################
#
# METODOS
#
###################################################

=head1 FUNCTIONS

Las funciones que aporta este mE<oacute>dulo son:

=head2 new

Uso: Constructor.

Entrada: Hash con los siguientes elementos:

=over 2

=item *

ruta_lisp (obligatorio): Ruta relativa al fichero con las fuentes
Lisp.

=item *

ruta_dem (obligatorio): Ruta relativa al fichero con la demostraciE<oacute>n
de ACL2.

=item *

deps (obligatorio): objeto ACL2::Dependencias con el grafo completo de
dependencias del fichero actual.

=item *

incrustar_dtd (opcional): si se halla presente y es distinto de 0,
se incrustarE<aacute> el contenido de la DTD en la salida XML.

=item *

destino_xml (opcional): si se halla presente, es la ruta relativa al
fichero XML en que se escribirE<aacute>n los resultados, o una referencia a la
cadena en que se escribirE<aacute>. De lo contrario, se mostrarE<aacute> por salida
estE<aacute>ndar.

=back

Salida: Referencia al nuevo objeto FiltroFicheros creado.

=cut

sub new 
{
    my ($class, %args) = @_;
    my $self = \%args;

    $self->{incrustar_dtd} = $self->{incrustar_dtd} || 0;

    bless($self, $class);
    return $self;
}

=head2 filtrar

Uso: Ejecuta el filtrado, escribiendo los resultados en el estado de
la clase, para su posterior devoluciE<oacute>n mediante get_writer.

=cut

sub filtrar
{
    my ($self) = @_;

    # Cargamos los dos ficheros
    my $guionLisp  = _volcarFichero($self->{ruta_lisp}) 
	or croak $lh->maketext("No se pudo abrir [_1]", $self->{ruta_lisp});
    my $salidaACL2 = _volcarFichero($self->{ruta_dem})
	or croak $lh->maketext("No se pudo abrir [_1]", $self->{ruta_dem});

    # Preparamos la salida XML
    $self->{writer} = _iniciarSalida($self->{incrustar_dtd}, $self->{destino_xml});

    # Creamos y lanzamos el procesador
    my $procesador = new ACL2::Procesador("lisp" => $guionLisp,
					  "acl2" => $salidaACL2,
					  "fuente_lisp" => $self->{ruta_lisp},
					  "fuente_dem" => $self->{ruta_dem},
					  "deps" => $self->{deps});
    $procesador->ejecutar($self->{writer});
    
    # Fin de salida
    _finalizarSalida($self->{writer});
}

=head2 get_salida

Uso: Devuelve la salida generada por el filtrado como una cadena con
XML. Si no hay una salida definida, se devuelve la cadena vacE<iacute>a.

Precondiciones: Se ha ejecutado el mE<eacute>todo filtrar.

=cut

sub get_salida
{
    my ($self) = @_;

    !defined($self->{destino_xml}) && return '';

    if (ref($self->{destino_xml}) ne '') {
	return ${$self->{destino_xml}};
    } else {
	open(SALIDA, '<', $self->{destino_xml});
	my $contenido = join '', <SALIDA>;
	close(SALIDA);
	return $contenido;
    }
}

##################################################
#
# FUNCIONES ESTATICAS PRIVADAS
#
##################################################

=head2 _iniciarSalida

Uso: Subrutina de utilidad.

Responsabilidad: Genera el inicio de la salida XML, hasta la declaraciE<oacute>n DOCTYPE.

ParE<aacute>metros:

=over 2

=item *

opt_incrustar_dtd - 0 si no se debe incrustar el DTD, != 0 de lo contrario.

=item *

opt_outfile - fichero de salida, o undef si se desea stdout.

=back

Devuelve: objeto XML::Writer a usar para la salida.

=cut

sub _iniciarSalida
{
    my ($opt_incrustar_dtd, $opt_outfile) = @_;

    # Generamos la salida XML
    my $writer;
    my $fich;
    
    if (defined $opt_outfile) {
	
	if (ref($opt_outfile) eq '') {
	    # Escalar con ruta a fichero XML destino
	    $fich = new IO::File(">${opt_outfile}") 
		or croak $lh->maketext("No se pudo abrir [_1]", $opt_outfile);
	} else {
	    # Referencia a cadena destino
	    $fich = $opt_outfile;
	}
	
	$writer = new XML::Writer(ENCODING => "utf-8", 
				  DATA_MODE => 1, 
				  DATA_INDENT => 2,
				  OUTPUT => $fich);
    } 
    else {
	$writer = new XML::Writer(ENCODING => "utf-8", 
				  DATA_MODE => 1, 
				  DATA_INDENT => 2);
    }
    
    $writer->xmlDecl();

    ##################################################
    #
    # DTD: INCRUSTADO/ENLAZADO
    #
    ##################################################
    
    # Ruta a la carpeta con la DTD
    my $dtd_carpeta = eval { dist_dir('ACL2-Procesador') } || '.';
    my $dtd_fichero = "script.dtd";

    # Vector con las posibles rutas al DTD
    my @rutasDTD = ("share/${dtd_fichero}",               # Durante pruebas
		    "$FindBin::Bin/share/${dtd_fichero}", # Durante desarrollo/PAR
		    "$dtd_carpeta/${dtd_fichero}",        # Tras instalación
		    );

    # Buscamos el DTD
    my $ruta_dtd;
    foreach my $ruta (@rutasDTD) {
	if (-r $ruta) {
	    $ruta_dtd = $ruta;
	    last;
	}
    }
    die $lh->maketext("No se halló el DTD: [_1]", ${ruta_dtd}) 
	if !defined($ruta_dtd);

    if ($opt_incrustar_dtd) {
	# Generamos DOCTYPE con DTD inline nosotros mismos (XML::Writer sólo
	# tiene soporte para públicos y de sistema)
	open(DTD_FILE, $ruta_dtd);
	my $dtd = join "", <DTD_FILE>;
	close(DTD_FILE);

	if (defined ($opt_outfile)) {
	    print $fich "<!DOCTYPE script [$dtd]>";
	}
	else {
	    print "<!DOCTYPE script [$dtd]>";
	}
    } 
    else {
	# Enlazamos con $dtd_fichero del directorio de ficheros compartidos
	$writer->doctype(ACL2::Tags::MAIN_TAG, undef, realpath($ruta_dtd));
    }
    
    return $writer;
    
}

=head2 _finalizarSalida

Uso: Subrutina de utilidad.

Responsabilidades: Marca el final de la salida XML (despuE<eacute>s
del tag que cierra al elemento raE<iacute>z).

ParE<aacute>metros: Ninguno.

Devuelve: Nada.

=cut

sub _finalizarSalida
{
    my ($writer) = @_;

    $writer->end();
}

=head2 _volcarFichero

Uso: Subrutina de utilidad.

Responsabilidades: Vuelca todo el contenido del fichero en sref.

ParE<aacute>metros: 

=over 2

=item *

ruta - ruta al fichero.

=item *

sref - referencia a cadena en la que volcar el fichero.

=back

Devuelve: 1 si se tuvo E<eacute>xito, 0 si no.

=cut

sub _volcarFichero {
    my ($ruta, $sref) = @_;
    open FICH, $ruta or return 0;
    $$sref .= join '', <FICH>;
}
